"""
MIT License with Additional Restrictions:

Copyright (c) 2024 Deviant

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

1. Redistributions of the Software in source code or any form must retain the
above copyright notice, this list of conditions and the following additional
restrictions.

2. Redistributions of the Software are not allowed to be modified or
distributed under a different name without prior written permission of the
copyright holder.

3. Redistributions of the Software or derivative works based on the Software
are not allowed to be sold or used for commercial purposes without prior
written permission of the copyright holder.

4. Redistributions of the Software or derivative works based on the Software
are not allowed to be copied, reproduced, or distributed without prior written
permission of the copyright holder.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""

__title__ = 'StarRailCard.py'
__author__ = 'DeviantUa'
__version__ = '2.1.9'
__license__ = 'MIT'
__copyright__ = 'Copyright 2024-present DeviantUa'

from .client import *
from .utils import *
from .src.tools.enums import *

