from .color_control import *
from .color import *
from .grandient_v1 import *
from .gradient_v2 import *
from .image_control import *
from .text_control import *
from .style_editor import *
from .diagrama import *